/*
** Hello Name Client
*/

#include <cbcp.h>
#define CBCP_NET_TCP_IMPLEMENTATION
#include <cbcp_net_tcp.h>

#include <stdio.h>
#include <string.h>



/*
** CBCP program
*/
int main(int argc, char const *argv[]) {
	if (argc < 2) {
		fprintf(stderr, "Usage:\n\t%s <cbcp_database_filepath>\n", argv[0]);
		return 1;
	}

	/*
	** Initialize CBCP by loading the database file for this host
	*/

	const char *cbcp_database_filepath = argv[1];
	CBCP_State *cbcp;

	cbcp_net_tcp_init(0);
	cbcp = cbcp_init(cbcp_database_filepath);

	if (cbcp == NULL) {
		fprintf(stderr, "Could not initialize CBCP.\n");
		return 1;
	}

	/*
	** Init commands
	*/

	CBCP_Command *command_hello_world;
	command_hello_world = cbcp_client_init_command(
		cbcp,
		"Server",
		"Hello",
		"Hello");

	if (!command_hello_world) {
		fprintf(stderr, "Could not initialize required command.\n");
		return 1;
	}

	/*
	** Client code here
	*/

	printf("Sending cbcp command to the server.\n");

	CBCP_Response response;
	char response_buffer[4096];

	response = cbcp_client_send_command(
		command_hello_world,
		NULL,
		0,
		response_buffer,
		sizeof(response_buffer));

	if(response.success)
	{
		printf("Server has received the command.\n");
		if(response.payload != NULL)
		{
			printf("Length of response: %d\n", response.payload_length);
			printf(
				"Response: %.*s\n",
				response.payload_length,
				(char*)response.payload);
		}
		else
		{
			printf("No payload?\n");
		}
	}
	else
		printf("Sending command to server was not successful!\n");

	return 0;
}

